/*
 * WBView.cpp
 * Part of the Workbar application
 * (c) 1999, Tim Vernum <zodsoft@kagi.com>
 * Distributed under the "Acknowledged Use Licence" see Doc/Licence
 */


#ifndef   WBVIEW_H
#include "WBView.h"
#endif

#ifndef   MESSAGES_H
#include "Messages.h"
#endif

#ifndef   COLOURS_H
#include "Colour.h"
#endif

#ifndef   GENERAL_H
#include "General.h"
#endif

#include <Roster.h>
#include <Window.h>
#include <PopUpMenu.h>
#include <MenuItem.h>

#include <stdio.h>

WorkBar :: View :: View( BRect r , bool horizontal )
:	BView( r , NULL, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE | B_PULSE_NEEDED )
,	fColours( NULL ) 
,	fHorizontal( horizontal )
,	fPopUp(NULL)
{
	SetViewColor( 224, 224, 224 ) ;
	SetupWorkspaces( ) ;
}

WorkBar:: View:: ~View( void )
{
	delete fPopUp ;
}

void WorkBar :: View :: Reset( float x , float y, bool h )
{
	fHorizontal = h ;
	ResizeTo( x , y ) ; 
}

void WorkBar :: View :: AttachedToWindow( void )
{
	SetFontSize( 8 ) ;
}

void WorkBar :: View :: SetupWorkspaces( void )
{
	delete[] fColours ;

	fnWorkspaces = count_workspaces( ) ;
	fColours = new Colours::RGBColour[ fnWorkspaces ] ;
	
	SetupCurrentWorkspace( ) ;	
}

void WorkBar :: View :: SetupCurrentWorkspace( void )
{
	fCurrentWorkspace = current_workspace( ) ;	
	fColours[ fCurrentWorkspace ] = ui_color( B_DESKTOP_COLOR ) ;
}

void WorkBar :: View :: MessageReceived( BMessage * msg )
{
	switch( msg->what )
	{
		case Messages::UpdateCurrentWorkspace:
		{
			SetupCurrentWorkspace() ;
			Invalidate() ;
			break ;
		}
		case Messages::UpdateAllWorkspaces:
		{
			SetupWorkspaces();
			Invalidate() ;
			break ;
		}

		case Messages::ResetView :
		{
			float x, y ;
			bool h ;
			msg->FindFloat( "width", &x ) ;
			msg->FindFloat( "height", &y ) ;
			msg->FindBool( "horizontal", &h ) ;
			Reset( x,y,h ) ;
			break ;
		}
		
		case B_SIMPLE_DATA:
		{
			entry_ref ref ;
			if( msg->WasDropped() )
			{
				ActivateWorkspace( ConvertFromScreen(msg->DropPoint()) ) ;
				int32 index = 0 ;
				while( msg->FindRef( "refs", index, &ref ) == B_OK )
				{
					LaunchRef( ref ) ;
					index++ ;
				}
			}
			else
			{
				inherited::MessageReceived( msg ) ;
			}
			break ;
		}
				
		default:
		{
			inherited::MessageReceived( msg ) ;
			break ;
		}
	}
}

void WorkBar :: View :: Draw( BRect r )
{
	if( !fnWorkspaces ) return ;

	r = Bounds() ;
	BRect drawRect = r ;
	float size ;
	if( fHorizontal )
	{
		size = ( r.Width() - (fnWorkspaces-1)*2 ) / fnWorkspaces ; 
		drawRect.right = size ;
	}
	else
	{
		size = ( r.Height() - (fnWorkspaces-1)*2 ) / fnWorkspaces ;
		drawRect.bottom = size ;
	}		
		
	BeginLineArray( fnWorkspaces * 4 ) ;
	
	uint32 i ;
	for( i=0 ; i< fnWorkspaces ; i++ )
	{
		DrawWorkspace( i, drawRect ) ;
		if( fHorizontal )
		{
			drawRect.left   = drawRect.right + 2 ;
			drawRect.right  = drawRect.left + size ;
		}		
		else
		{
			drawRect.top    = drawRect.bottom + 2 ;
			drawRect.bottom = drawRect.top + size ;
		}
	}

	EndLineArray() ;
}

void WorkBar :: View :: DrawWorkspace( uint32 i, BRect r )
{
	SetLowColor( fColours[i] ) ;

	BRect box = r;

	// Draw Background colour
	if( i == fCurrentWorkspace )
	{
		SetHighColor( fColours[i].IPolarise() ) ;
		FillRect( r, B_SOLID_HIGH ) ;
		
		box.left   += r.Width()/5 ;
		box.right  -= r.Width()/5 ;
		box.top    += r.Height()/6 ;
		box.bottom -= r.Height()/6 ;

		FillRect( box, B_SOLID_LOW ) ;
	}
	else
	{
		FillRect( r, B_SOLID_LOW ) ;
	}

	// Draw Button Frame
	const rgb_color& col1	= ( i != fCurrentWorkspace )
							? Colours::White
							: Colours::Black ;
	const rgb_color& col2	= ( i != fCurrentWorkspace )
							? Colours::BeLightShadow
							: Colours::BeDarkShadow ;

	BPoint 	a = box.LeftTop() ,
			b = box.RightTop() ;
				 
	AddLine( a , b , col1 ) ;
	b = box.LeftBottom( ) ;
	AddLine( a , b , col1 ) ;
	
	b.x += 1 ;
	a = box.RightBottom() ;
	AddLine( a , b , col2 ) ;

	b = box.RightTop() ;
	b.y += 1 ;
	AddLine( a , b , col2 ) ;

	// Draw Text
	font_height f_h ;
	GetFontHeight( &f_h ) ;
	float fh = f_h.ascent + f_h.descent + f_h.leading ;

	if( r.Height() > fh )
	{
		// Set Text Colour
		SetHighColor( fColours[i].IPolarise() ) ;		
		
		char str[5] ;
		sprintf( str, "%lu", (i+1) ) ;

		float w = StringWidth( str ) ;
		BPoint pt( r.left + r.Width()/2 - w/2 , r.top + r.Height()/2 + fh/2 - 1 ) ;

		DrawString( str, pt ) ;
	}	
}
				
void  WorkBar :: View :: MouseDown( BPoint pt )
{
	int32 buttons ;
	this->Window()->CurrentMessage()->FindInt32("buttons", &buttons ) ;
	if( buttons & B_PRIMARY_MOUSE_BUTTON )
		ActivateWorkspace( pt ) ;
	else if( buttons & B_SECONDARY_MOUSE_BUTTON )
		RaisePopup( pt ) ;
	else 
		this->Window()->PostMessage( Messages::Options , this->Window() ) ;
}

int32 WorkBar :: View :: WorkspaceAt( BPoint pt )
{
	BRect r = Bounds() ;
	float size = ( fHorizontal ? r.Width() : r.Height() ) / fnWorkspaces ;

	int32 i = static_cast<int32>( ( fHorizontal ? pt.x : pt.y ) / size ) ;

	if( i < 0 || i >= static_cast<int32>(fnWorkspaces) )
		return -1 ;

	return i; 
}	

void  WorkBar :: View :: ActivateWorkspace( BPoint pt )
{
	if( count_workspaces() != static_cast<int32>(fnWorkspaces) )
	{
		SetupWorkspaces() ;
		Invalidate() ;
	}
	else
	{
#if 0
		WorkBar::Window	* window = dynamic_cast<WorkBar::Window *>( Window() ) ;
		if( window )
			window->SavePosition() ;
#endif

		int32 ws = WorkspaceAt( pt ) ;
		if ( ws != -1 )
			activate_workspace( ws ) ;
	}
}

void WorkBar :: View :: LaunchRef( entry_ref& ref ) 
{
	BEntry ent( &ref ) ;
	if( !ent.IsDirectory() )
	{
		be_roster->Launch( &ref ) ;
		return ;
	}
	
	// Tell Tracker to Open Directory
	BMessage send( B_REFS_RECEIVED ) ;
	BMessenger tracker("application/x-vnd.Be-TRAK" );
	if ( !tracker.IsValid() )
	        return ;
	
	send.AddRef("refs", &ref );
	tracker.SendMessage(&send) ;
}

void WorkBar :: View :: RaisePopup( BPoint pt )
{
	ConvertToScreen( &pt ) ;

	if( !fPopUp )
	{
		fPopUp = new BPopUpMenu( "menu" ) ;
		fPopUp->AddItem( new BMenuItem( "Options", new BMessage( Messages::Options ) ) ) ;		
		fPopUp->AddItem( new BMenuItem( "About", new BMessage( B_ABOUT_REQUESTED ) ) ) ;
		fPopUp->AddItem( new BMenuItem( "Quit", new BMessage( B_QUIT_REQUESTED ) ) ) ;
	}
	
	BMenuItem * menu = fPopUp -> Go ( pt, false, false, false ) ;
	if( menu )
		Looper()->PostMessage( menu->Message()->what ) ;
}

WorkBar :: View :: mouse_state :: mouse_state( void )
:	where(0,0)
,	when(0)
,	transit( B_OUTSIDE_VIEW )
{
}

void WorkBar :: View ::MouseMoved( BPoint pt, uint32 transit, const BMessage * msg )
{
	mouse.where = pt ;
	mouse.when  = system_time() ;
	mouse.transit = transit ;
	mouse.dragging = (msg != NULL) ;
}

void WorkBar :: View :: Pulse( void )
{
	if( mouse.dragging && (mouse.transit == B_ENTERED_VIEW || mouse.transit == B_INSIDE_VIEW ) )
	{
		bigtime_t now = system_time() ;
		if( now - mouse.when >= 1000000 )
		{
			ActivateWorkspace( mouse.where ) ;
		}
	}
}
